/****************************************************************************
*   Mac-Vocalizer: Predict speech articulation program                      *
*   Copyright Paul Williamson, KB5MU 2002                                   *
*   Last update: October 18, 2002                                           *
*   Version: 1.0.0                                                          *
*                                                                           *
*   Interface derived from that of:                                         *
*                  VOCALIZER: A speech articulation program                 *
*               Copyright John A. Magliacane, KD2BD 1999-2000               *
*                         Last update: 27-Jul-2002                          *
*****************************************************************************
* This program is used by PREDICT to annunciate satellite coordinates       *
* through the Macintosh speech synthesizer.                                 *
* Since it takes quite a while to output the speech, this program should    *
* normally be called as a background process.                               *
*****************************************************************************
*                                                                           *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by the     *
* Free Software Foundation; either version 2 of the License or any later    *
* version.                                                                  *
*                                                                           *
* This program is distributed in the hope that it will useful, but WITHOUT  *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     *
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License     *
* for more details.                                                         *
*                                                                           *
*****************************************************************************/

#import <Foundation/Foundation.h>
#import <Carbon/Carbon.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>

/* Callback function for when the speech is done */
void done_speaking(SpeechChannel chan, SInt32 refCon)
{
    kill(getpid(), SIGUSR1); // Signal to interrupt the long sleep
}

/* Do-nothing function to catch the signal sent by the callback */
void do_nothing(int foo)
{
}

int main(int argc,char **argv)
{
    SpeechChannel  spch;
    char           text[200];

    /* Check the arguments before setting up the speech channel */
    if (argc < 3  || argc > 4)
        exit(-1);

    /* Set up a signal handler to intercept the signal sent by the callback.
       If we don't do this, the default signal handler will do something silly,
       like print "User Signal 1" on the console. */
    signal(SIGUSR1, &do_nothing);
  
    /* Configure the speech channel */
    NewSpeechChannel(NULL, &spch);          // NULL means use default voice
    if (spch == NULL)
        exit(-1);
        
    /* Set up the callback for when the speech is done */
    SetSpeechInfo(spch, soSpeechDoneCallBack, &done_speaking);

    /* Create the string to be spoken, inserting azimuth and elevation */
    sprintf(text,
            "this is predict. satellite is at %s degrees azimuth and %s degrees elevation",
            argv[1], argv[2]);

    if (argc==4)
        {
        if (argv[3][0] == '+')
            strcat(text, " and approaching.");
        else if (argv[3][0] == '-')
            strcat(text, " and receding.");
    }
    else
        strcat(text, ".");
            
    // printf("Saying \"%s\"\n", text);
          
    /* Finally, speak the message. */
    SpeakText(spch, text, strlen(text));
        
    /* Wait for it to be done, as indicated by a signal from
       the callback function (which interrupts a sleep() call) */
    sleep(30); // upper bound duration, expect to be awakened sooner.
        
    /* Clean up the speech channel */
    DisposeSpeechChannel(spch);

    return 0;
}
